@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.48 2000/01/12 22:48:16 nisse Exp $
+ * $Id: channel.h,v 1.49 2000/01/26 22:10:15 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -238,7 +238,8 @@
        (error_code . UINT32)))
 */
 
-struct exception *make_channel_open_exception(UINT32 error_code, char *msg);
+struct exception *
+make_channel_open_exception(UINT32 error_code, const char *msg);
 
 
 /* GABA:
